@@ -41,6 +41,7 @@
 #include "jstk.h"
 #include "jstk_hw.h"
 #include "jstk_axis.h"
+#include "jstk_key.h"
 #include "jstk_options.h"
 
 
@@ -294,6 +295,7 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
                 return !Success;
             }
         }
+        jstkInitKeys(pJstk, priv);
 
         m = 2;
         for (i=0; i<MAXAXES; i++) 
@@ -322,7 +324,6 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
                                    1, /* resolution */
                                    0, /* min_res */
                                    1); /* max_res */
-
             for (i=0; i<MAXAXES; i++) 
                 if (priv->axis[i].type != TYPE_NONE)
             {
@@ -353,6 +354,8 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
 
     case DEVICE_OFF:
     case DEVICE_CLOSE:
+        if (!pJstk->public.on)
+            break;
         DBG(1, ErrorF("jstkDeviceControlProc  what=%s\n", 
             (what == DEVICE_CLOSE) ? "CLOSE" : "OFF"));
 
@@ -417,7 +420,7 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     local->fd = -1;
     local->dev = NULL;
     local->private = priv;
-    local->type_name = "JOYSTICK";
+    local->type_name = XI_MOUSE;
     local->history_size = 0;
     local->always_core_feedback = 0;
     local->conf_idev = dev;
@@ -431,7 +434,10 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->keys_enabled = TRUE;
     priv->amplify = 1.0f;
     priv->buttonmap.size = 0;
-    priv->keymap.size = 0;
+    priv->keymap.size = 1;
+    priv->keymap.map[0] = NoSymbol;
+    priv->repeat_delay = 0;
+    priv->repeat_interval = 0;
 
     /* Initialize default mappings */
     for (i=0; i<MAXAXES; i++) {
@@ -465,12 +471,24 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->button[2].mapping      = MAPPING_BUTTON;
     priv->button[2].buttonnumber = 3;
 
-    /* Two axes by default */
+    /* First two axes are a stick for moving */
     priv->axis[0].type      = TYPE_BYVALUE;
     priv->axis[0].mapping   = MAPPING_X;
     priv->axis[1].type      = TYPE_BYVALUE;
     priv->axis[1].mapping   = MAPPING_Y;
 
+    /* Next two axes are a stick for scrolling */
+    priv->axis[2].type      = TYPE_BYVALUE;
+    priv->axis[2].mapping   = MAPPING_ZX;
+    priv->axis[3].type      = TYPE_BYVALUE;
+    priv->axis[3].mapping   = MAPPING_ZY;
+
+    /* Next two axes are a pad for moving */
+    priv->axis[4].type      = TYPE_ACCELERATED;
+    priv->axis[4].mapping   = MAPPING_X;
+    priv->axis[5].type      = TYPE_ACCELERATED;
+    priv->axis[5].mapping   = MAPPING_Y;
+
     priv->buttonmap.scrollbutton[0] = jstkGetButtonNumberInMap(priv, 4);
     priv->buttonmap.scrollbutton[1] = jstkGetButtonNumberInMap(priv, 5);
     priv->buttonmap.scrollbutton[2] = jstkGetButtonNumberInMap(priv, 6);
@@ -504,6 +522,22 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     }
 #endif
 
+    /* Parse option for autorepeat */
+    if ((s = xf86SetStrOption(dev->commonOptions, "AutoRepeat", NULL))) {
+        int delay, rate;
+        if (sscanf(s, "%d %d", &delay, &rate) != 2) {
+            xf86Msg(X_ERROR, "%s: \"%s\" is not a valid AutoRepeat value", 
+                    local->name, s);
+        } else {
+            priv->repeat_delay = delay;
+            if (rate != 0)
+                priv->repeat_interval = 1000/rate;
+            else priv->repeat_interval = 0;
+            DBG(1, xf86Msg(X_CONFIG, "Autorepeat set to delay=%d, interval=%d\n",
+                           priv->repeat_delay,priv->repeat_interval));
+        }
+        xfree(s);
+    }
 
     /* Process button mapping options */
     for (i=0; i<MAXBUTTONS; i++) {
@@ -523,7 +557,7 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         sprintf(p,"MapAxis%d",i+1);
         s = xf86SetStrOption(dev->commonOptions, p, NULL);
         if (s != NULL) {
-            jstkParseAxisOption(s, &priv->axis[i], local->name);
+            jstkParseAxisOption(s, priv, &priv->axis[i], local->name);
         }
         DBG(1, xf86Msg(X_CONFIG, 
                        "Axis %d type is %d, mapped to %d, amplify=%.3f\n", i+1, 
@@ -533,7 +567,7 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     }
 
     /* return the LocalDevice */
-    local->flags |= XI86_CONFIGURED ;
+    local->flags |= XI86_CONFIGURED;
 
     return (local);
 
@@ -612,7 +646,6 @@ jstkDriverPlug(pointer  module,
  * jstkDriverUnplug --
  *
  * Called when the driver is unloaded
- * NOTICE: When does this actually happen? What needs to be cleaned up?
  *
  ***************************************************************************
  */
